import requests
import pandas as pd
import io


def elevation(lat=(40, 41), lon=(-65, -60)):
    """
    Function to request data from SRTM30.
    Bathymetry / Topography (SRTM30) is a global bathymetry/topography data product distributed by the USGS EROS data
    center. The data product has a resolution of 30 seconds (roughly 1 km).

    Keyword Arguments:
        lat (tuple): with min and max latitude
        lon (tuple): with min and max longitude

    Returns:
        ElevationSurface object.

        - Attributes:
            df (pandas dataframe): lat, lon and elev columns

        - Methods:
            plot: generate a 3D surface plot
    """

    return ElevationSurface(lat, lon)


def point_elev(lat, lon):
    minlat = lat - 0.001
    maxlat = lat + 0.001
    minlon = lon - 0.001
    maxlon = lon + 0.001
    c = elevation(lat=(minlat, maxlat), lon=(minlon, maxlon))
    znew = round(c.df.elev.mean())

    return znew


class ElevationSurface(object):
    """
    3D surface object.

    Attributes:
        df (pandas dataframe): lat, lon and elev columns

    Methods:
        plot: generate a 3D surface plot
    """
    def __init__(self, lat, lon):

        # Define the domain of interest
        minlat = lat[0]
        maxlat = lat[1]
        minlon = lon[0]
        maxlon = lon[1]

        url = 'http://coastwatch.pfeg.noaa.gov/erddap/griddap/usgsCeSrtm30v6.csv?topo[(' + str(maxlat) + '):1:(' + \
              str(minlat) + ')][(' + str(minlon) + '):1:(' + str(maxlon) + ')]'
        s = requests.get(url).content
        df = pd.read_csv(io.StringIO(s.decode('utf-8')), header=0)
        df.reset_index(drop=True)
        df.drop(0, inplace=True)
        df = df[['latitude', 'longitude', 'topo']].astype(float)
        df.rename(columns={'latitude': 'lat', 'longitude': 'lon', 'topo': 'elev'}, inplace=True)

        self.df = df

    def plot(self):

        import plotly.graph_objects as go
        from numpy import array, unique

        lon_u = unique(self.df.lon)
        lat_u = unique(self.df.lat)

        elev = array([self.df.elev[i: i + len(lon_u)] for i in range(0, len(self.df.elev), len(lon_u))])

        fig = go.Figure(data=[go.Surface(z=elev, x=lon_u, y=lat_u)])
        fig.update_layout(scene=dict(
            xaxis_title='Lon, °',
            yaxis_title='Lat, °',
            zaxis_title='Elevation, m'))

        return fig



import numpy as np
from plotly.subplots import make_subplots
import plotly.graph_objects as go


def plot_log(logs, title=None):
    """
    Function to plot well logs data using matplotlib.

    Arguments:
        logs (dataframe): each log is a column

    Keyword Arguments:
        title (str or None): text to add as title if required

    Returns:
        plotly figure: plot
    """

    logs.dropna(axis=0, how='any', inplace=True)

    logs.reset_index(inplace=True)
    logs = logs.replace({np.nan: None})

    names = list(logs.columns)
    if 'DEPT' in names:
        logs["Depth"] = logs["DEPT"]
        names.pop(names.index('DEPT'))
    else:
        logs["Depth"] = list(logs.index)
    logs = logs.sort_values(by='Depth')

    fig = make_subplots(rows=1, cols=len(names),
                        subplot_titles=names)
    col = 1
    for x in names:
        fig.add_trace(
            go.Scatter(x=logs[x], y=logs.Depth),
            row=1, col=col
        )
        col += 1

    fig.update_layout(height=600, width=800, showlegend=False)

    if title is not None:
        fig.update_layout(title_text=title)

    return fig



import requests
import pandas as pd
import numpy as np


def wellbore(dataset_number):
    """
    Function to request wellbore data from NPD Database.

    Arguments:
        dataset_number (int): to select the dataset
            (1: oil samples,
            2: NPD ID,
            3: lithostratigraphy,
            4: history,
            5: drilling mud,
            6: drill stem tests,
            7: documents,
            8: cores,
            9: core photos,
            10: coordinates,
            11: casing and leak off,
            12: exploration,
            13: development,
            14: shallow)

    Returns:
        dataframe: requested data
    """

    data_list = ['with-oil-samples', 'with-npdid', 'with-lithostratigraphy', 'with-history', 'with-drilling-mud',
                 'with-drill-stem-tests', 'with-documents', 'with-cores', 'with-core-photos', 'with-coordinates',
                 'with-casing-and-leak-off', 'exploration', 'development', 'shallow']

    data = data_list[dataset_number - 1]

    return get_dataset(data)


def get_dataset(data):
    """
    looping through all pages, append in JSON dictionary, create dataframe
    This function is based on the Anne Estoppey's code https://github.com/AnneEstoppey/EasyDataTools
    """
    dataset = requests.get('https://hotell.difi.no/api/json/npd/wellbore/' + data + '?page=1').json()

    url_dataset = 'http://hotell.difi.no/api/json/npd/wellbore/' + data

    pages = dataset['pages']
    data_list = []
    for i in range(pages):
        url = url_dataset + f'?page={i + 1}'
        dataset_final = requests.get(url).json()
        entries = len(dataset_final['entries'])

        for entry in range(entries):
            data_list.append(dataset_final['entries'][entry])

    df = pd.DataFrame(data_list)
    df.replace('', np.nan, inplace=True)
    return df
